"""
Utilities Module
----------------
Various utilities that are useful when interacting with Goated state.
- Price tick bucketing for various price formats
- Conversion tools
- Decimal numeric data encoders
- Payoff Signature helpers (for more information on PayoffSignatures see main documentation)

"""