"""
Goated Package
--------------
The Goated Python package contains wrappers for the Goated platform API and tools for handling local copies of state.
For more information on the Goated exchange and to create or manage your account please visit https://goated.com/

"""

from .version import __version__
from .client import *
from .state import *
from .instructions import *
from .utils import *